@font-face{font-family:"Open Sans";font-style:normal;font-weight:400;font-display:swap;src:local("Open Sans Regular"),local("OpenSans-Regular"),url(/static/media/OpenSans-Regular.833b3033.woff2) format("woff2")}@font-face{font-family:"Open Sans";font-style:normal;font-weight:600;font-display:swap;src:local("Open Sans SemiBold"),local("OpenSans-SemiBold"),url(/static/media/OpenSans-SemiBold.d76f9a5e.woff2) format("woff2")}@font-face{font-family:"Source Code Pro";font-style:normal;font-weight:400;font-display:swap;src:local("Source Code Pro Regular"),local("SourceCodePro-Regular"),url(/static/media/SourceCodePro-Regular.6b0680a0.woff2) format("woff2")}@font-face{font-family:"Source Code Pro";font-style:normal;font-weight:500;font-display:swap;src:local("Source Code Pro Medium"),local("SourceCodePro-Medium"),url(/static/media/SourceCodePro-Medium.0de41603.woff2) format("woff2")}@font-face{font-family:"Source Code Pro";font-style:normal;font-weight:700;font-display:swap;src:local("Source Code Pro Bold"),local("SourceCodePro-Bold"),url(/static/media/SourceCodePro-Bold.9efc4535.woff2) format("woff2")}
/*# sourceMappingURL=main.af0c7fcc.chunk.css.map */